@@ -54,7 +54,8 @@ def run():
                       help="whether to allow non maxfiltered data (BOOLEAN)",
                       action='store_true')
     parser.add_option('--line_freq', dest='line_freq',
-                      help="The frequency of the line noise (e.g. 50 or 60).")
+                      help="The frequency of the line noise in Hz "
+                           "(e.g. 50 or 60). If unknown, pass None")
 
     opt, args = parser.parse_args()
 
@@ -75,11 +76,12 @@ def run():
                     hsp=opt.hsp, config=opt.config,
                     allow_maxshield=opt.allow_maxshield)
     if opt.line_freq is not None:
-        raw.info['line_freq'] = opt.line_freq
+        line_freq = None if opt.line_freq == "None" else opt.line_freq
+        raw.info['line_freq'] = line_freq
     write_raw_bids(raw, bids_path, event_id=opt.event_id,
                    events_data=opt.events_data, overwrite=opt.overwrite,
                    verbose=True)
 
 
-if __name__ == '__main__':  # pragma: no cover
+if __name__ == '__main__':
     run()
