@@ -68,6 +68,12 @@ def test_raw_to_bids(tmpdir):
                      '--line_freq', 60)):
         mne_bids_raw_to_bids.run()
 
+    # Test line_freq == 'None'
+    with ArgvSetter(('--subject_id', subject_id, '--task', task, '--raw',
+                     raw_fname, '--bids_root', output_path,
+                     '--line_freq', 'None', '--overwrite', 1)):
+        mne_bids_raw_to_bids.run()
+
     # Test EDF files as well
     edf_data_path = op.join(base_path, 'edf', 'tests', 'data')
     edf_fname = op.join(edf_data_path, 'test.edf')
