@@ -18,13 +18,12 @@ import os
 import os.path as op
 import re
 import shutil as sh
-from datetime import datetime
 
 from scipy.io import loadmat, savemat
 
+import mne
 from mne.io import (read_raw_brainvision, read_raw_edf, read_raw_bdf,
                     anonymize_info)
-from mne.utils import check_version
 
 from mne_bids.path import BIDSPath, _parse_ext, _mkdir_p
 from mne_bids.utils import _get_mrk_meas_date, _check_anonymize
@@ -452,44 +451,23 @@ def copyfile_edf(src, dest, anonymize=None):
         if len(rec_info) == 0 or len(rec_info.split(' ')) != 5:
             rec_info = "Startdate X X X X"
         pid, sex, birthdate, name = id_info.split(' ')
-        startdate, admin_code, tech, equip = rec_info.split(' ')[1:5]
+        start_date, admin_code, tech, equip = rec_info.split(' ')[1:5]
 
-        # Try to anonymize recording date using 4-digit year from
-        # "Startdate" field from "local recording identification"
-        # section, because the generic "startdate" field only
-        # supports 2-digit years.
-        #
-        # XXX: recording dates are specified twice in EDF/EDF+/BDF,
-        # MNE-Python currently parses the 2-digit year field. If that is
-        # changed to defaulting to the 4-digit field, this try-except
-        # clause can be removed. See:
-        # https://github.com/mne-tools/mne-python/issues/8544
-        try:
-            true_year = datetime.strptime(startdate, '%d-%b-%Y').year
-            newdate = raw.info['meas_date'].replace(year=true_year)
-            raw.info['meas_date'] = newdate
-        except ValueError as e:
-            # We could not parse the "Startdate" field from "local recording
-            # identification" section (e.g., because it was "X").
-            # So fall back to using the standard "startdate" field that only
-            # supports 2-digit years.
-            # This is what MNE-Python currently parses and puts into
-            # raw.info["meas_date"]
-            if "does not match format '%d-%b-%Y'" not in str(e):
-                raise e
+        # Try to anonymize the recording date
         daysback, keep_his = _check_anonymize(anonymize, raw, '.edf')
-        info = anonymize_info(raw.info, daysback=daysback, keep_his=keep_his)
-        startdate = datetime.strftime(info['meas_date'], '%d-%b-%Y').upper()
-        meas_date = datetime.strftime(info['meas_date'], '%d.%m.85')
+        anonymize_info(raw.info, daysback=daysback, keep_his=keep_his)
+        start_date = '01-JAN-1985'
+        meas_date = '01.01.85'
 
         # Anonymize ID info and write to file
         if keep_his:
             # Always remove participant birthdate and name to be safe
             id_info = [pid, sex, "X", "X"]
-            rec_info = ["Startdate", startdate, admin_code, tech, equip]
+            rec_info = ["Startdate", start_date, admin_code, tech, equip]
         else:
             id_info = ["0", "X", "X", "X"]
-            rec_info = ["Startdate", startdate, "X", "mne-bids_anonymize", "X"]
+            rec_info = ["Startdate", start_date, "X",
+                        "mne-bids_anonymize", "X"]
         with open(dest, 'r+b') as f:
             f.seek(8)  # id_info field starts 8 bytes in
             f.write(bytes(" ".join(id_info).ljust(80), 'ascii'))
@@ -521,7 +499,7 @@ def copyfile_eeglab(src, dest):
     copyfile_kit
 
     """
-    if not check_version('scipy', '1.5.0'):  # pragma: no cover
+    if not mne.utils.check_version('scipy', '1.5.0'):  # pragma: no cover
         raise ImportError('SciPy >=1.5.0 is required handling EEGLAB data.')
 
     # Get extenstion of the EEGLAB file
@@ -567,7 +545,7 @@ def copyfile_bti(raw, dest):
 
     Parameters
     ----------
-    raw : instance of Raw
+    raw : mne.io.Raw
         An MNE-Python raw object of BTi data.
     dest : str | pathlib.Path
         Destination to copy the BTi data to.
