@@ -20,6 +20,7 @@ from mne_bids.config import (
     ALLOWED_PATH_ENTITIES, ALLOWED_FILENAME_EXTENSIONS,
     ALLOWED_FILENAME_SUFFIX, ALLOWED_PATH_ENTITIES_SHORT,
     ALLOWED_DATATYPES, SUFFIX_TO_DATATYPE, ALLOWED_DATATYPE_EXTENSIONS,
+    ALLOWED_SPACES,
     reader, ENTITY_VALUE_TYPE)
 from mne_bids.utils import (_check_key_val, _check_empty_room_basename,
                             param_regex, _ensure_tuple)
@@ -176,7 +177,11 @@ class BIDSPath(object):
     recording : str | None
         The recording name for this item. Corresponds to "rec".
     space : str | None
-        The coordinate space for an anatomical file. Corresponds to "space".
+        The coordinate space for an anatomical or sensor position
+        files (e.g., ``*_electrodes.tsv``, ``*_markers.mrk``).
+        Corresponds to "space".
+        Note that valid values for ``space`` must come from a list
+        of BIDS keywords as described in the BIDS specification.
     split : int | None
         The split of the continuous recording file for ``.fif`` data.
         Corresponds to "split".
@@ -187,7 +192,7 @@ class BIDSPath(object):
         'meg', 'markers', 'eeg', 'ieeg', 'T1w',
         'participants', 'scans', 'electrodes', 'coordsystem',
         'channels', 'events', 'headshape', 'digitizer',
-        'behav', 'phsyio', 'stim'
+        'beh', 'physio', 'stim'
     extension : str | None
         The extension of the filename. E.g., ``'.json'``.
     datatype : str
@@ -198,8 +203,7 @@ class BIDSPath(object):
         The root for the filename to be created. E.g., a path to the folder
         in which you wish to create a file with this name.
     check : bool
-        If True enforces the entities to be valid according to the
-        current BIDS standard. Defaults to True.
+        If ``True``, enforces BIDS conformity. Defaults to ``True``.
 
     Attributes
     ----------
@@ -219,15 +223,12 @@ class BIDSPath(object):
     fpath : pathlib.Path
         The full file path.
     check : bool
-        If ``True``, enforces the entities to be valid according to the
-        BIDS specification. The check is performed on instantiation
-        and any ``update`` function calls (and may be overridden in the
-        latter).
+        Whether to enforce BIDS conformity.
 
     Examples
     --------
     >>> bids_path = BIDSPath(subject='test', session='two', task='mytask',
-                                 suffix='ieeg', extension='.edf')
+                             suffix='ieeg', extension='.edf')
     >>> print(bids_path.basename)
     sub-test_ses-two_task-mytask_ieeg.edf
     >>> bids_path
@@ -235,7 +236,7 @@ class BIDSPath(object):
     basename: sub-test_ses-two_task-mytask_ieeg.edf)
     >>> # copy and update multiple entities at once
     >>> new_bids_path = bids_path.copy().update(subject='test2',
-                                                   session='one')
+                                                session='one')
     >>> print(new_bids_path.basename)
     sub-test2_ses-one_task-mytask_ieeg.edf
     >>> # printing the BIDSPath will show relative path when
@@ -257,6 +258,23 @@ class BIDSPath(object):
     /bids_dataset/sub-test2/ses-one/ieeg/sub-test2_ses-one_task-mytask_ieeg.edf
     >>> print(new_bids_path.directory)
     /bids_dataset/sub-test2/ses-one/ieeg/
+
+    Notes
+    -----
+    BIDS entities are generally separated with a ``"_"`` character, while
+    entity key/value pairs are separated with a ``"-"`` character.
+    There are checks performed to make sure that there are no ``'-'``, ``'_'``,
+    or ``'/'`` characters contained in any entity keys or values.
+
+    To represent a filename such as ``dataset_description.json``,
+    one can set ``check=False``, and pass ``suffix='dataset_description'``
+    and ``extension='.json'``.
+
+    ``BIDSPath`` can also be used to represent file and folder names of data
+    types that are not yet supported through MNE-BIDS, but are recognized by
+    BIDS. For example, one can set ``datatype`` to ``dwi`` or ``func`` and
+    pass ``check=False`` to represent diffusion-weighted imaging and
+    functional MRI paths.
     """
 
     def __init__(self, subject=None, session=None,
@@ -371,7 +389,7 @@ class BIDSPath(object):
 
         Returns
         -------
-        bidspath : instance of BIDSPath
+        bidspath : mne_bids.BIDSPath
             The copied bidspath.
         """
         return deepcopy(self)
@@ -387,7 +405,7 @@ class BIDSPath(object):
 
         Returns
         -------
-        self : instance of BIDSPath
+        self : mne_bids.BIDSPath
             The BIDSPath object.
         """
         self.directory.mkdir(parents=True, exist_ok=exist_ok)
@@ -481,25 +499,27 @@ class BIDSPath(object):
 
         Also performs error checks on various entities to
         adhere to the BIDS specification. Specifically:
-        - ``suffix`` should be one of: ``anat``, ``eeg``, ``ieeg``, ``meg``
+        - ``datatype`` should be one of: ``anat``, ``eeg``, ``ieeg``, ``meg``
         - ``extension`` should be one of the accepted file
         extensions in the file path: ``.con``, ``.sqd``, ``.fif``,
         ``.pdf``, ``.ds``, ``.vhdr``, ``.edf``, ``.bdf``, ``.set``,
         ``.edf``, ``.set``, ``.mef``, ``.nwb``
-        - ``suffix`` should be one acceptable file suffixes in: ``meg``,
+        - ``suffix`` should be one of the acceptable file suffixes in: ``meg``,
         ``markers``, ``eeg``, ``ieeg``, ``T1w``,
         ``participants``, ``scans``, ``electrodes``, ``channels``,
         ``coordsystem``, ``events``, ``headshape``, ``digitizer``,
-        ``behav``, ``phsyio``, ``stim``
+        ``beh``, ``physio``, ``stim``
+        - Depending on the modality of the data (EEG, MEG, iEEG),
+        ``space`` should be a valid string according to Appendix VIII
+        in the BIDS specification.
 
         Parameters
         ----------
         check : None | bool
-            If a boolean, controls whether to enforce the entities to be valid
-            according to the BIDS specification. This will set the
-            ``.check`` attribute accordingly. If ``None``, rely on the existing
-            ``.check`` attribute instead, which is set upon ``BIDSPath``
-            instantiation. Defaults to ``None``.
+            If a boolean, controls whether to enforce BIDS conformity. This
+            will set the ``.check`` attribute accordingly. If ``None``, rely on
+            the existing ``.check`` attribute instead, which is set upon
+            `mne_bids.BIDSPath` instantiation. Defaults to ``None``.
         **kwargs : dict
             It can contain updates for valid BIDS path entities:
             'subject', 'session', 'task', 'acquisition', 'processing', 'run',
@@ -508,7 +528,7 @@ class BIDSPath(object):
 
         Returns
         -------
-        bidspath : instance of BIDSPath
+        bidspath : mne_bids.BIDSPath
             The updated instance of BIDSPath.
 
         Examples
@@ -527,13 +547,18 @@ class BIDSPath(object):
         >>> print(bids_path.basename)
         sub-test_ses-two_acq-test_ieeg.vhdr
         """
+        # Update .check attribute
+        if check is not None:
+            self.check = check
+
         for key, val in kwargs.items():
             if key == 'root':
                 _validate_type(val, types=('path-like', None), item_name=key)
                 continue
 
             if key == 'datatype':
-                if val is not None and val not in ALLOWED_DATATYPES:
+                if val is not None and val not in ALLOWED_DATATYPES \
+                        and self.check:
                     raise ValueError(f'datatype ({val}) is not valid. '
                                      f'Should be one of '
                                      f'{ALLOWED_DATATYPES}')
@@ -559,18 +584,23 @@ class BIDSPath(object):
         extension = kwargs.get('extension')
         if extension is not None:
             if not extension.startswith('.'):
-                extension = f'.{extension}'
-                kwargs['extension'] = extension
+                kwargs['extension'] = f'.{extension}'
 
         # error check entities
         for key, val in kwargs.items():
+
             # check if there are any characters not allowed
             if val is not None and key != 'root':
-                _check_key_val(key, val)
+                if key == 'suffix' and not self.check:
+                    # suffix may skip a check if check=False to allow
+                    # things like "dataset_description.json"
+                    pass
+                else:
+                    _check_key_val(key, val)
 
             # set entity value, ensuring `root` is a Path
-            if key == 'root' and val is not None:
-                val = Path(val)
+            if val is not None and key == 'root':
+                val = Path(val).expanduser()
             setattr(self, key, val)
 
         # infer datatype if suffix is uniquely the datatype
@@ -578,21 +608,27 @@ class BIDSPath(object):
                 self.suffix in SUFFIX_TO_DATATYPE:
             self.datatype = SUFFIX_TO_DATATYPE[self.suffix]
 
-        # Update .check attribute and perform a check of the entities.
-        if check is not None:
-            self.check = check
+        # Perform a check of the entities.
         self._check()
         return self
 
-    def match(self):
+    def match(self, check=False):
         """Get a list of all matching paths in the root directory.
 
         Performs a recursive search, starting in ``.root`` (if set), based on
         `BIDSPath.entities` object. Ignores ``.json`` files.
 
+        Parameters
+        ----------
+        check : bool
+            If ``True``, only returns paths that conform to BIDS. If ``False``
+            (default), the ``.check`` attribute of the returned
+            `mne_bids.BIDSPath` object will be set to ``True`` for paths that
+            do conform to BIDS, and to ``False`` for those that don't.
+
         Returns
         -------
-        bids_paths : list of BIDSPath
+        bids_paths : list of mne_bids.BIDSPath
             The matching paths.
         """
         if self.root is None:
@@ -628,8 +664,25 @@ class BIDSPath(object):
             fpath = list(self.root.rglob(f'*{fname}*'))[0]
             datatype = _infer_datatype_from_path(fpath)
 
+            # to check whether the BIDSPath is conforming to BIDS if
+            # check=True, we first instantiate without checking and then run
+            # the check manually, allowing us to be more specific about the
+            # exception to catch
             bids_path = BIDSPath(root=self.root, datatype=datatype,
-                                 extension=extension, **entities)
+                                 extension=extension, check=False,
+                                 **entities)
+
+            bids_path.check = True
+
+            try:
+                bids_path._check()
+            except ValueError:
+                # path is not BIDS-compatible
+                if check:  # skip!
+                    continue
+                else:
+                    bids_path.check = False
+
             bids_paths.append(bids_path)
 
         return bids_paths
@@ -659,6 +712,26 @@ class BIDSPath(object):
                                      f'allowed. Use one of these extensions '
                                      f'{ALLOWED_FILENAME_EXTENSIONS}.')
 
+            # labels from space entity must come from list (appendix VIII)
+            space = self.space
+            if space is not None:
+                datatype = getattr(self, 'datatype', None)
+                if datatype is None:
+                    raise ValueError('You must define datatype if you want to '
+                                     'use space in your BIDSPath.')
+
+                allowed_spaces_for_dtype = ALLOWED_SPACES.get(datatype, None)
+                if allowed_spaces_for_dtype is None:
+                    raise ValueError(f'space entity is not valid for datatype '
+                                     f'{self.datatype}')
+                elif space not in allowed_spaces_for_dtype:
+                    raise ValueError(f'space ({space}) is not valid for '
+                                     f'datatype ({self.datatype}).\n'
+                                     f'Should be one of '
+                                     f'{allowed_spaces_for_dtype}')
+                else:
+                    pass
+
             # error check suffix
             suffix = self.suffix
             if suffix is not None and \
@@ -903,7 +976,7 @@ def get_entities_from_fname(fname, on_error='raise'):
 
     Parameters
     ----------
-    fname : BIDSPath | str
+    fname : mne_bids.BIDSPath | str
         The path to parse.
     on_error : 'raise' | 'warn' | 'ignore'
         If any unsupported labels in the filename are found and this is set
@@ -986,7 +1059,7 @@ def _find_matching_sidecar(bids_path, suffix=None,
 
     Parameters
     ----------
-    bids_path : BIDSPath
+    bids_path : mne_bids.BIDSPath
         Full name of the data file
     suffix : str | None
         The filename suffix. This is the entity after the last ``_``
